# Generated by roxygen2: do not edit by hand

S3method("$",DBItest_tweaks)
S3method(format,DBItest_tweaks)
S3method(print,DBItest_tweaks)
export(get_default_context)
export(make_context)
export(set_default_context)
export(test_all)
export(test_arrow)
export(test_compliance)
export(test_connection)
export(test_driver)
export(test_getting_started)
export(test_meta)
export(test_result)
export(test_some)
export(test_sql)
export(test_stress)
export(test_transaction)
export(tweaks)
import(DBI)
import(rlang)
import(testthat, except = c(is_null, is_false, is_true))
importFrom(callr,r)
importFrom(lubridate,with_tz)
importFrom(magrittr,"%>%")
importFrom(methods,extends)
importFrom(methods,findMethod)
importFrom(methods,getClass)
importFrom(methods,getClasses)
importFrom(methods,hasMethod)
importFrom(methods,is)
importFrom(methods,new)
importFrom(stats,setNames)
importFrom(utils,head)
importFrom(withr,with_output_sink)
importFrom(withr,with_temp_libpaths)
