#' @details
#' The two most important functions are [make_context()] and
#' [test_all()].  The former tells the package how to connect to your
#' DBI backend, the latter executes all tests of the test suite. More
#' fine-grained test functions (all with prefix `test_`) are available.
#'
#' See the package's vignette for more details.
#'
#' @author Kirill Müller
"_PACKAGE"
