# Created by create_generics(), do not edit by hand
all_dbi_generics <- function() {
  c(
    "Id",
    "dbAppendTable",
    "dbBegin",
    "dbBind",
    "dbCanConnect",
    "dbClearResult",
    "dbColumnInfo",
    "dbCommit",
    "dbConnect",
    "dbCreateTable",
    "dbDataType",
    "dbDisconnect",
    "dbExecute",
    "dbExistsTable",
    "dbFetch",
    "dbGetInfo",
    "dbGetQuery",
    "dbGetRowCount",
    "dbGetRowsAffected",
    "dbGetStatement",
    "dbHasCompleted",
    "dbIsReadOnly",
    "dbIsValid",
    "dbListFields",
    "dbListObjects",
    "dbListTables",
    "dbQuoteIdentifier",
    "dbQuoteLiteral",
    "dbQuoteString",
    "dbReadTable",
    "dbRemoveTable",
    "dbRollback",
    "dbSendQuery",
    "dbSendStatement",
    "dbUnquoteIdentifier",
    "dbWithTransaction",
    "dbWriteTable",
    NULL
  )
}
