% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/test-compliance.R
\name{test_compliance}
\alias{test_compliance}
\title{Test full compliance to DBI}
\usage{
test_compliance(skip = NULL, run_only = NULL, ctx = get_default_context())
}
\arguments{
\item{skip}{\verb{[character()]}\cr A vector of regular expressions to match
against test names; skip test if matching any.
To improve precision, the regular expressions are matched against the
entire test name minus a possible suffix \verb{_N} where \code{N} is a number.
For example, \code{skip = "exists_table"} will skip both
\code{"exists_table_1"} and \code{"exists_table_2"}, but not \code{"there_exists_table"}.}

\item{run_only}{\verb{[character()]}\cr A vector of regular expressions to match
against test names; run only these tests.
The regular expressions are matched against the entire test name.}

\item{ctx}{\verb{[DBItest_context]}\cr A test context as created by
\code{\link[=make_context]{make_context()}}.}
}
\description{
Test full compliance to DBI
}
\seealso{
Other tests:
\code{\link[=test_arrow]{test_arrow()}},
\code{\link[=test_connection]{test_connection()}},
\code{\link[=test_driver]{test_driver()}},
\code{\link[=test_getting_started]{test_getting_started()}},
\code{\link[=test_meta]{test_meta()}},
\code{\link[=test_result]{test_result()}},
\code{\link[=test_sql]{test_sql()}},
\code{\link[=test_stress]{test_stress()}},
\code{\link[=test_transaction]{test_transaction()}}
}
\concept{tests}
