% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spec-meta-get-statement.R
\docType{data}
\name{spec_meta_get_statement}
\alias{spec_meta_get_statement}
\title{spec_meta_get_statement}
\value{
\code{dbGetStatement()} returns a string, the query used in
either \code{\link[=dbSendQuery]{dbSendQuery()}}
or \code{\link[=dbSendStatement]{dbSendStatement()}}.
}
\description{
spec_meta_get_statement
}
\section{Failure modes}{

Attempting to query the statement for a result set cleared with
\code{\link[=dbClearResult]{dbClearResult()}} gives an error.
}

\seealso{
Other meta specifications: 
\code{\link{spec_get_info}},
\code{\link{spec_meta_bind}},
\code{\link{spec_meta_column_info}},
\code{\link{spec_meta_get_row_count}},
\code{\link{spec_meta_get_rows_affected}},
\code{\link{spec_meta_has_completed}},
\code{\link{spec_meta_is_valid}}
}
\concept{meta specifications}
