% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spec-meta-bind-.R
\name{make_placeholder_fun}
\alias{make_placeholder_fun}
\title{Create a function that creates n placeholders}
\usage{
make_placeholder_fun(pattern)
}
\arguments{
\item{pattern}{\verb{[character(1)]}\cr Any character, optionally followed by \code{1} or \code{name}. Examples: \code{"?"}, \code{"$1"}, \code{":name"}}
}
\value{
\verb{[function(n)]}\cr A function with one argument \code{n} that
returns a vector of length \code{n} with placeholders of the specified format.
}
\description{
For internal use by the \code{placeholder_format} tweak.
}
\examples{
body(DBItest:::make_placeholder_fun("?"))
DBItest:::make_placeholder_fun("?")(2)
DBItest:::make_placeholder_fun("$1")(3)
DBItest:::make_placeholder_fun(":name")(5)
}
\keyword{internal}
