% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/test-stress.R
\name{test_stress}
\alias{test_stress}
\title{Stress tests}
\usage{
test_stress(skip = NULL, ctx = get_default_context())
}
\arguments{
\item{skip}{\verb{[character()]}\cr A vector of regular expressions to match
against test names; skip test if matching any.
The regular expressions are matched against the entire test name
minus a possible suffix \verb{_N} where \code{N} is a number.
For example, \code{skip = "exists_table"} will skip both
\code{"exists_table_1"} and \code{"exists_table_2"}.}

\item{ctx}{\verb{[DBItest_context]}\cr A test context as created by
\code{\link[=make_context]{make_context()}}.}
}
\description{
Stress tests
}
\seealso{
Other tests: 
\code{\link{test_arrow}()},
\code{\link{test_compliance}()},
\code{\link{test_connection}()},
\code{\link{test_driver}()},
\code{\link{test_getting_started}()},
\code{\link{test_meta}()},
\code{\link{test_result}()},
\code{\link{test_sql}()},
\code{\link{test_transaction}()}
}
\concept{tests}
\keyword{internal}
