% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spec-connection-disconnect.R
\docType{data}
\name{spec_connection_disconnect}
\alias{spec_connection_disconnect}
\title{spec_connection_disconnect}
\value{
\code{dbDisconnect()} returns \code{TRUE}, invisibly.
}
\description{
spec_connection_disconnect
}
\section{Failure modes}{

A warning is issued on garbage collection when a connection has been
released without calling \code{dbDisconnect()},
but this cannot be tested automatically.
A warning is issued immediately when calling \code{dbDisconnect()} on an
already disconnected
or invalid connection.
}

\seealso{
Other connection specifications: 
\code{\link{spec_get_info}}
}
\concept{connection specifications}
