% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spec-compliance-methods.R
\docType{data}
\name{spec_compliance_methods}
\alias{spec_compliance_methods}
\title{spec_compliance_methods}
\description{
spec_compliance_methods
}
\section{DBI classes and methods}{

A backend defines three classes,
which are subclasses of
\linkS4class{DBIDriver},
\linkS4class{DBIConnection},
and \linkS4class{DBIResult}.
The backend provides implementation for all methods
of these base classes
that are defined but not implemented by DBI.
All methods defined in \pkg{DBI} are reexported (so that the package can
be used without having to attach \pkg{DBI}),
and have an ellipsis \code{...} in their formals for extensibility.
}

\concept{compliance specifications}
