% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spec-result-create-table-with-data-type.R
\docType{data}
\name{spec_result_create_table_with_data_type}
\alias{spec_result_create_table_with_data_type}
\title{spec_result_create_table_with_data_type}
\description{
spec_result_create_table_with_data_type
}
\section{Specification}{

All data types returned by \code{dbDataType()} are usable in an SQL statement
of the form
\code{"CREATE TABLE test (a ...)"}.
}

\seealso{
Other result specifications: 
\code{\link{spec_result_clear_result}},
\code{\link{spec_result_execute}},
\code{\link{spec_result_fetch}},
\code{\link{spec_result_get_query}},
\code{\link{spec_result_roundtrip}},
\code{\link{spec_result_send_query}},
\code{\link{spec_result_send_statement}}
}
\concept{result specifications}
