% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spec-getting-started.R
\docType{data}
\name{spec_getting_started}
\alias{spec_getting_started}
\title{spec_getting_started}
\description{
spec_getting_started
}
\section{Definition}{

A DBI backend is an R package
which imports the \pkg{DBI}
and \pkg{methods}
packages.
For better or worse, the names of many existing backends start with
\sQuote{R}, e.g., \pkg{RSQLite}, \pkg{RMySQL}, \pkg{RSQLServer}; it is up
to the backend author to adopt this convention or not.
}

\concept{getting specifications}
