% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/test-result.R
\name{test_result}
\alias{test_result}
\title{Test the "Result" class}
\usage{
test_result(skip = NULL, run_only = NULL, ctx = get_default_context())
}
\arguments{
\item{skip}{\verb{[character()]}\cr A vector of regular expressions to match
against test names; skip test if matching any.
The regular expressions are matched against the entire test name.}

\item{run_only}{\verb{[character()]}\cr A vector of regular expressions to match
against test names; run only these tests.
The regular expressions are matched against the entire test name.}

\item{ctx}{\verb{[DBItest_context]}\cr A test context as created by
\code{\link[=make_context]{make_context()}}.}
}
\description{
Test the "Result" class
}
\seealso{
Other tests: 
\code{\link{test_compliance}()},
\code{\link{test_connection}()},
\code{\link{test_driver}()},
\code{\link{test_getting_started}()},
\code{\link{test_meta}()},
\code{\link{test_sql}()},
\code{\link{test_stress}()},
\code{\link{test_transaction}()}
}
\concept{tests}
